package core

import (
	"git.akyoto.dev/cli/q/src/errors"
	"git.akyoto.dev/cli/q/src/scope"
	"git.akyoto.dev/cli/q/src/token"
	"git.akyoto.dev/cli/q/src/x86"
)

// ResolveTypes parses the input and output types.
func (f *Function) ResolveTypes() error {
	for i, param := range f.Input {
		param.Name = param.tokens[0].Text(f.File.Bytes)
		typeName := param.tokens[1:].Text(f.File.Bytes)
		param.Type = f.TypeByName(typeName)

		if param.Type == nil {
			return errors.New(&errors.UnknownType{Name: typeName}, f.File, param.tokens[1].Position)
		}

		uses := token.Count(f.Body, f.File.Bytes, token.Identifier, param.Name)

		if uses == 0 && param.Name != "_" {
			return errors.New(&errors.UnusedVariable{Name: param.Name}, f.File, param.tokens[0].Position)
		}

		f.AddVariable(&scope.Variable{
			Name:     param.Name,
			Type:     param.Type,
			Register: x86.InputRegisters[i],
			Alive:    uses,
		})
	}

	for _, param := range f.Output {
		typeName := param.tokens.Text(f.File.Bytes)
		param.Type = f.TypeByName(typeName)

		if param.Type == nil {
			return errors.New(&errors.UnknownType{Name: typeName}, f.File, param.tokens[1].Position)
		}
	}

	return nil
}
